#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pywkdb.utils.db_base import DbBase
from pywkdb.utils.db_utils import DBUtils
from pywkdb.utils.sql_builder import SqlBuilder
from pywkdb.utils.url_parse import UrlParse



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pywkdb.db.db_mysql import MySQLDb
from pywkdb.db.db_mssql import MsSQLDb
from pywkdb.db.db_sqlite import SqliteDb
from pywkdb.db.db_oraclel import OracleDb


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from pywkdb.utils.db_base import DbBase
from pywkdb.utils.url_parse import UrlParse
logger = logging.getLogger('MySQLDb')


class MySQLDb(DbBase):

    def __init__(self, url, with_tran=False):
        """
        构造函数,初始化数据库连接
        :param url: example mssql://sa:123456@127.0.0.1:1443/dbname?charset=utf-8
        :param with_tran:
        """
        super(MySQLDb, self).__init__(url, with_tran)

    def connect(self, url_parse=None):
        """
        生成连接串
        :return:
        """
        if url_parse is None:
            url_parse = UrlParse(self._url)
        return self._pymysql().connect(
            host=url_parse.hostname,
            user=url_parse.username,
            password=url_parse.password,
            database=url_parse.path,
            port=url_parse.port
        )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

class SqlBuilder(object):

    INSERT_SQL = 1
    UPDATE_SQL = 2
    DELETE_SQL = 3
    SELECT_SQL = 4

    def __init__(self, table_name):
        self._tablename = table_name
        self._data_dic: dict = {}
        self._where_dic: dict = {}
        self._where_sql = ''
        self._where_sql_params: tuple = ()
        self._select_fields: tuple = ()

    def new_pool(self):
        self._data_dic = {}
        self._where_dic = {}
        self._where_sql = ''
        self._where_sql_params = ()
        self._select_fields = ()

    def data(self, data: dict):
        if isinstance(data, list) or isinstance(data, set) or isinstance(data, tuple):
            data = dict(data)
        self._data_dic = data

    def add_data(self, data: dict):
        self._data_dic = SqlBuilder._merge_dicts(self._data_dic, data)

    def put_data(self, key, value):
        self._data_dic[key] = value

    def where(self, datas: dict):
        self.add_where(datas)
        #self._where_dic = data

    def put_where(self, key, value):
        self.add_where_equals(key, value)
        #self._where_dic[key] = value

    def add_where(self, datas: dict):
        if isinstance(datas, list) or isinstance(datas, set) or isinstance(datas, tuple):
            datas = dict(datas)
        for key in datas.keys():
            self.add_where_equals(key, datas.get(key))
        # self._where_dic = SqlBuilder._merge_dicts(self._where_dic, data)

    def add_where_in(self, key, data, is_match=True):
        if not data or len(data) <= 0:
            raise RuntimeError('data is Null')
        if is_match:
            self._add_sql('{key} is in ({instr}) '.format(key=key, instr=','.join(['%s'] * len(data))), data)
        else:
            self._add_sql('{key} is not ({instr}) '.format(key=key, instr=','.join(['%s'] * len(data))), data)

    def add_where_isnull(self, key, is_match=True):
        if is_match:
            self._add_sql('{key} is NULL '.format(key=key))
        else:
            self._add_sql('{key} is not NULL '.format(key=key))

    def add_where_like(self, key, data, is_match=True):
        if not data or data is None:
            raise RuntimeError('data is Null')
        if is_match:
            self._add_sql('{key} like %s '.format(key=key), data)
        else:
            self._add_sql('{key} not like %s '.format(key=key), data)

    def add_where_between(self, key, value1=None, value2=None, is_match=True):
        if is_match:
            if value1 is None:
                self._add_sql('{key} < %s '.format(key=key), (value1,))
            elif value1 is None:
                self._add_sql('{key} > %s '.format(key=key), (value1,))
            else:
                self._add_sql('{key} BETWEEN %s AND %s '.format(key=key), (value1, value2))
        else:
            self._add_sql('{key} not BETWEEN %s AND %s '.format(key=key), (value1, value2))

    def add_where_equals(self, key, data=None, equals=True):
        if equals:
            self._add_sql('{key} = %s '.format(key=key), data)
        else:
            self._add_sql('{key} <> %s '.format(key=key), data)

    def add_where_greater_than(self, key, data, is_equal=False):
        if not data or data is None:
            raise RuntimeError('data is Null')
        if is_equal:
            self._add_sql('{key} >= %s '.format(key=key), data)
        else:
            self._add_sql('{key} > %s '.format(key=key), data)

    def add_where_less_than(self, key, data, is_equal=False):
        if not data or data is None:
            raise RuntimeError('data is Null')
        if is_equal:
            self._add_sql('{key} <= %s '.format(key=key), data)
        else:
            self._add_sql('{key} < %s '.format(key=key), data)

    def add_where_sql(self, sql, data=()):
        self._add_sql(sql, data)

    def get(self, key) -> dict:
        if key in self._data_dic:
            return self._data_dic[key]
        return None

    def _add_sql(self, sql_str, data=()):
        if sql_str is not None:
            if self._where_sql != '':
                self._where_sql += " AND "
            self._where_sql += sql_str
        if data is not None:
            if isinstance(data, tuple):
                self._where_sql_params = self._where_sql_params + data
            elif isinstance(data, list):
                self._where_sql_params = self._where_sql_params + tuple(data)
            else:
                self._where_sql_params = self._where_sql_params + (data,)

    @staticmethod
    def _merge_dicts(x, y) -> dict:
        return {**x, **y}

    @property
    def tablename(self) -> str:
        return self._tablename

    def _get_wsql(self) -> str:
        if len(self._where_dic) <= 0 and self._where_sql == '':
            return ''
        if len(self._where_dic) > 0 and self._where_sql != '':
            return 'WHERE {wsql} AND {wsql2} '.format(
                wsql='AND '.join(["{key} = %s".format(key=key) for key in self._where_dic]),
                wsql2=self._where_sql
            )
        elif len(self._where_dic) > 0:
            return 'WHERE {wsql} '.format(
                wsql='AND '.join(["{key} = %s".format(key=key) for key in self._where_dic]),
            )
        else:
            return 'WHERE {wsql2}'.format(wsql2=self._where_sql)
        return sql_where

    def select_field(self, field=None):
        if isinstance(field, tuple):
            self._select_fields = self._select_fields + field
        elif isinstance(field, set):
            self._select_fields = self._select_fields + tuple(list(set(field)))
        elif isinstance(field, list):
            self._select_fields = self._select_fields + tuple(field)
        else:
            self._select_fields = self._select_fields + (field,)

    def get_db_type(self):
        from pywkdb.db.db_mssql import MsSQLDb
        from pywkdb.db.db_mysql import MySQLDb
        from pywkdb.db.db_sqlite import SqliteDb
        db_type = 1
        try:
            if self._sql_db:
                if isinstance(self._sql_db, MySQLDb) or isinstance(self._sql_db, SqliteDb):
                    db_type = 1
                elif isinstance(self._sql_db, MsSQLDb):
                    db_type = 2
        except Exception as e:
            db_type = 1
        return db_type

    def _get_select_sql(self, limit=None, row_num=None) -> (str, tuple):
        if self._select_fields is None or len(self._select_fields) <= 0:
            self._select_fields = ('*',)
        sql = 'SELECT {fields_str} FROM {tables} {whersql}'.format(
            fields_str=','.join(["{key}".format(key=key) for key in self._select_fields]),
            tables=self._tablename,
            whersql=self._get_wsql()
        )
        if limit:
            sql = '{sql} limit {limit} '.format(sql=sql,limit=limit)
        if row_num:
            if limit:
                sql = '{sql} , {row_num} '.format(sql=sql,row_num=row_num)
            else:
                sql = '{sql} limit {limit}, {row_num} '.format(sql=sql,limit=limit,row_num = row_num)
        sql_params = tuple(self._data_dic.values()) + tuple(self._where_dic.values()) + self._where_sql_params
        return sql, sql_params

    def _get_insert_sql(self, allow_null=False) -> (str, tuple):
        if self._data_dic is None or len(self._data_dic) <= 0:
            raise RuntimeError('Data dic is Null')
        if allow_null:
            del_keys = ()
            for key in self._data_dic.keys():
                if self._data_dic.get(key) is None:
                    del_keys = del_keys + (key,)
                    #self._data_dic.pop(key)
            for key in del_keys:
                self._data_dic.pop(key)
        sql = 'INSERT INTO {tables} ({keys}) VALUES({values})'.format(
            tables=self._tablename,
            keys=','.join(self._data_dic.keys()),
            values=','.join(['%s'] * len(self._data_dic))
        )
        sql_params = tuple(self._data_dic.values())
        return sql, sql_params

    def _get_update_sql(self) -> (str, tuple):
        if self._data_dic is None or len(self._data_dic) <= 0:
            raise RuntimeError('Data dic is Null')
        sql = 'UPDATE {tables} SET {values} {whersql}'.format(
            tables=self._tablename,
            values=','.join(["{key} = %s".format(key=key) for key in self._data_dic]),
            whersql=self._get_wsql()
        )
        sql_params = tuple(self._data_dic.values()) + tuple(self._where_dic.values()) + self._where_sql_params
        return sql, sql_params

    def _get_delete_sql(self) -> (str, tuple):
        sql = 'DELETE TABLE {tables} {whersql}'.format(
            tables=self._tablename,
            values=','.join(["{key} = %s".format(key=key) for key in self._data_dic]),
            whersql=self._get_wsql()
        )
        sql_params = tuple(self._where_dic.values()) + self._where_sql_params
        return sql, sql_params

    def build(self, sql_type=SELECT_SQL, limit=None, row_num=None) -> (str, tuple):
        if sql_type == self.INSERT_SQL:
            return self._get_insert_sql()
        elif sql_type == self.UPDATE_SQL:
            return self._get_update_sql()
        elif sql_type == self.DELETE_SQL:
            return self._get_delete_sql()
        elif sql_type == self.SELECT_SQL:
            return self._get_select_sql(limit, row_num)


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pywkmisc import get_config
from pywkdb.utils.sql_builder import SqlBuilder
from pywkdb.utils.db_base import DbBase
from pywkdb.utils.url_parse import UrlParse
import pandas as pd


class DBUtils(SqlBuilder):

    def __init__(self, table_name, db_base: DbBase = None, db_str: str = None):
        self._sql_db = None
        super(DBUtils, self).__init__(table_name)
        if db_base is not None:
            self._sql_db = db_base

        if db_base is None and db_str is not None:
            self._sql_db = DbBase(db_str)

    def insert(self, sql, params=None):
        if isinstance(sql, tuple):
            sql, params = sql
        return self._sql_db.insert(sql, params)

    def execute(self, sql, params=None) -> int:
        if isinstance(sql, tuple):
            sql, params = sql
        return self._sql_db.update(sql, params)

    def select_sql(self, sql, params=None) -> (tuple, list):
        if isinstance(sql, tuple):
            sql, params = sql
        return self._sql_db.select(sql, params)

    def select_sql_list(self, sql, params=None) -> list:
        if isinstance(sql, tuple):
            sql, params = sql
        return self._sql_db.select_data(sql, params)

    def select_sql_frame_data(self, sql, params=None) -> pd.DataFrame:
        if isinstance(sql, tuple):
            sql, params = sql
        return self._sql_db.select_frame_data(sql, params)

    def save(self, allow_null=False):
        return self.insert(self._get_insert_sql(allow_null))

    def update(self) -> int:
        return self.execute(self.build(self.UPDATE_SQL))

    def delete(self) -> int:
        return self.execute(self.build(self.DELETE_SQL))

    def select(self, limit=None, row_num=None) -> (tuple, list):
        return self.select_sql(self.build(self.SELECT_SQL, limit, row_num))

    def select_list(self, limit=None, row_num=None) -> list:
        return self.select_sql_list(self.build(self.SELECT_SQL, limit, row_num))

    def select_frame_data(self, limit=None, row_num=None) -> pd.DataFrame:
        return self.select_sql_frame_data(self.build(self.SELECT_SQL, limit, row_num))

    def __del__(self):
        if self._sql_db is None:
            del self._sql_db

    @staticmethod
    def get_db(db_str=None):
        if db_str is None:
            db_str = get_config('db')
        """
        mysql://root:123456@127.0.0.1:3306/dbname?charset=utf-8
        mssql://sa:123456@127.0.0.1:1443/dbname?charset=utf-8
        sqlite://:/C:/work/python/data/dbfile.db
        :param db_str:
        :return:
        """
        url_parse = UrlParse(db_str)
        db_base = None
        if url_parse.scheme == 'mysql':
            from pywkdb.db.db_mysql import MySQLDb
            db_base = MySQLDb(db_str)
        elif url_parse.scheme == 'mssql':
            from pywkdb.db.db_mssql import MsSQLDb
            db_base = MsSQLDb(db_str)
        elif url_parse.scheme == 'oracle':
            from pywkdb.db.db_oraclel import OracleDb
            db_base = OracleDb(db_str)
        elif url_parse.scheme == 'sqlite':
            from pywkdb.db.db_sqlite import SqliteDb
            db_base = SqliteDb(db_str)
        db_base.connect(url_parse)
        return db_base


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from kombu.utils.url import url_to_parts


class UrlParse(object):

    def __init__(self, url):
        self._scheme, self._host, self._port, self._user, self._password, self._path, self._query = url_to_parts(url)

    @property
    def scheme(self) -> str:
        return self._scheme

    @property
    def hostname(self) -> str:
        return self._host

    @property
    def port(self) -> int:
        return self._port

    @property
    def username(self) -> str:
        return self._user

    @property
    def password(self) -> str:
        return self._password

    @property
    def path(self) -> str:
        return self._path

    @property
    def query(self) -> dict:
        return self._query

    def __str__(self):
        import json
        return json.dumps({
            "scheme": self.scheme,
            "hostname": self.hostname,
            "port": self.port,
            "username": self.username,
            "password": self.password,
            "path": self.path,
            "query": self.query
        }, indent=2)


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import pandas as pd
from .url_parse import UrlParse

logger = logging.getLogger('DbBase')


class DbBase(object):
    
    def __init__(self, url, with_tran=False):
        self._url = url
        self._with_tran = with_tran
        self._conn = None
        if self._with_tran:
            self._conn = self.connect()
    
    def _get_conn(self):
        if not self._with_tran:
            return self.connect()
        else:
            return self._conn
    
    @staticmethod
    def _execute(conn, sql: str, params=None):
        """
        内部方法，执行sql代码
        :param sql:     sql代码
        :param params:  传递参数
        :return:        游标值Cursor
        """
        cur = conn.cursor()
        logger.info(sql)
        logger.info(params)
        if params is None:
            cur.execute(sql)
        else:
            cur.execute(sql, params)
        return cur
    
    def update(self, sql: str, params=None):
        """
        更新数据库
        :param sql:     sql代码
        :param params:  传递参数
        :return:        操作影响记录数
        """
        cur = None
        conn = None
        try:
            conn = self._get_conn()
            cur = self._execute(conn, sql, params)
            conn.commit()
            return cur.rowcount
        except Exception as e:
            logger.error("update_error:{}".format(e))
        finally:
            self.close(cur)
            if not self._with_tran:
                self.close(conn)
    
    def fetchmany(self, fetch_number, sql: str, params=None):
        """
        获取指定数据量，根据游标向下获取
        :param fetch_number:    数据条数
        :param sql:             sql代码
        :param params:          传递参数
        :return:                记录集
        """
        cur = None
        conn = None
        try:
            conn = self._get_conn()
            cur = self._execute(conn, sql, params)
            return cur.fetchmany(fetch_number)
        except Exception as e:
            logger.error("fetchmany_error:{}".format(e))
        finally:
            self.close(cur)
            if not self._with_tran:
                self.close(conn)
    
    def select(self, sql: str, params=None) -> (tuple, list):
        """
        用来查询表数据
        :return: 返回数据和表头
        """
        cur = None
        conn = None
        try:
            conn = self._get_conn()
            cur = self._execute(conn, sql, params)
            col = cur.description
            results = cur.fetchall()
            # 执行结果转化为dataframe
            headers = []
            for i in range(len(col)):
                headers.append(col[i][0])
            return results, headers
        except Exception as e:
            logger.error("fetchmany_error:{}".format(e))
        finally:
            self.close(cur)
            if not self._with_tran:
                self.close(conn)
    
    def select_frame_data(self, sql: str, params=None) -> pd.DataFrame:
        """
        查询结果返回DataFrame格式。
        example
        df = self.select_frame_data()
        for index, row in df.iterrows():
            print(index,row)
        :return: 返回DataRame格式数据
        """
        result, headers = self.select(sql, params)
        return pd.DataFrame(list(result), columns=headers)
    
    def select_data(self, sql: str, params=None) -> list:
        """
        查询结果返回List
        :param sql:
        :param params:
        :return:
        """
        results, headers = self.select(sql, params)
        items = []
        for result in results:
            item = {}
            i = 0
            for header in headers:
                item[header] = result[i]
                i += 1
            items.append(item)
        return items
    
    @staticmethod
    def close(conn):
        """
        关闭游标或数据库
        :param conn: 游标或数据库链接
        """
        if conn is not None:
            try:
                conn.close()
                del conn
            except Exception as e:
                logger.error("close:{}".format(e))
    
    def __del__(self):
        """
        删除对象时，自动释放数据库连接
        :return:
        """
        logger.debug('del sqldb')
        if self._conn is not None:
            try:
                self.close(self._conn)
            except Exception as e:
                logger.error("__del__:{}".format(e))
    
    def insert(self, sql: str, params=None):
        """
        插入一条记录
        :param sql:     sql代码
        :param params:  传递参数
        :return:        主键ID
        """
        cur = None
        conn = None
        try:
            conn = self._get_conn()
            cur = self._execute(conn, sql, params)
            try:
                insert_id = conn.insert_id()
            except:
                try:
                    insert_id = cur.lastrowid
                except:
                    print('无法获取id')
            conn.commit()
            return insert_id
        except Exception as e:
            conn.rollback()
            logger.error(e)
            return -1
        finally:
            self.close(cur)
            if not self._with_tran:
                self.close(conn)
    
    def connect(self, url_parse=None):
        """
        生成连接串
        :return:
        """
        raise RuntimeError('找不到对应的数据配置,目前支持mysql、mssql、sqlite 和 oracle')

    @staticmethod
    def _pymysql():
        try:
            import pymysql
            return pymysql
        except ImportError:
            raise RuntimeError('PyMySQL不存在，pip install PyMySQL')
    
    @staticmethod
    def _pymssql():
        try:
            import pymssql
            return pymssql
        except ImportError:
            raise RuntimeError('pymssql不存在，pip install pymssql')

    @staticmethod
    def _cxoracle():
        try:
            import cx_Oracle
            return cx_Oracle
        except ImportError:
            raise RuntimeError('cx_Oracle不存在，pip install cx_Oracle')

    @staticmethod
    def _sqlite3():
        try:
            import sqlite3
            return sqlite3
        except ImportError:
            raise RuntimeError('SqliteDb不存在')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from pywkdb.utils.db_base import DbBase
from pywkdb.utils.url_parse import UrlParse
logger = logging.getLogger('OracleDb')


class OracleDb(DbBase):

    def __init__(self, url, with_tran=False):
        """
        构造函数,初始化数据库连接
        :param url: example oracle://user:passwd@127.0.0.1:1521/orcl
                oracle://user:passwd@127.0.0.1:1521/orcl?NLS_LANG=SIMPLIFIED CHINESE_CHINA.UTF8
        :param with_tran:
        """
        super(OracleDb, self).__init__(url, with_tran)

    def connect(self, url_parse=None):
        """
        生成连接串
        :return:
        """
        if url_parse is None:
            url_parse = UrlParse(self._url)

        import cx_Oracle
        import os
        if len(url_parse.query) > 0:
            for query in url_parse.query:
                os.environ[query] = url_parse.query[query]
        return self._cxoracle().connect(
            url_parse.username,
            url_parse.password,
            cx_Oracle.makedsn(url_parse.hostname, url_parse.port, url_parse.path)
        )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from pywkdb.utils.db_base import DbBase
from pywkdb.utils.url_parse import UrlParse
logger = logging.getLogger('SqliteDb')


class SqliteDb(DbBase):

    def __init__(self, url, with_tran=False):
        """
        构造函数,初始化数据库连接
        :param url: example sqlite://:/d:/data/dbfile.db
                             sqlite://://root/data/dbfile.db
        :param with_tran:
        """
        super(SqliteDb, self).__init__(url, with_tran)


    def connect(self, url_parse=None):
        """
        生成连接串
        :return:
        """
        if url_parse is None:
            url_parse = UrlParse(self._url)
        return self._sqlite3().connect(url_parse.path)

    def insert(self, sql: str, params=None):
        """
        插入一条记录
        :param sql:     sql代码
        :param params:  传递参数
        :return:        主键ID
        """
        cur = None
        conn = None
        try:
            conn = self._get_conn()
            cur = self._execute(conn, sql, params)
            conn.commit()
            return cur.lastrowid
        except Exception as e:
            logger.error("insert_error:{}".format(e))
        finally:
            self.close(cur)
            if not self._with_tran:
                self.close(conn)


if __name__ == '__main__':
    pass
